<?php
/*
Plugin Name: WPC Linked Variation for WooCommerce
Plugin URI: https://wpclever.net/
Description: WPC Linked Variation built to link separate products together by attributes.
Version: 3.4.2
Author: WPClever
Author URI: https://wpclever.net
Text Domain: wpc-linked-variation
Domain Path: /languages/
Requires at least: 4.0
Tested up to: 6.1
WC requires at least: 3.0
WC tested up to: 7.2
*/

defined( 'ABSPATH' ) || exit;

! defined( 'WPCLV_VERSION' ) && define( 'WPCLV_VERSION', '3.4.2' );
! defined( 'WPCLV_FILE' ) && define( 'WPCLV_FILE', __FILE__ );
! defined( 'WPCLV_URI' ) && define( 'WPCLV_URI', plugin_dir_url( __FILE__ ) );
! defined( 'WPCLV_DIR' ) && define( 'WPCLV_DIR', plugin_dir_path( __FILE__ ) );
! defined( 'WPCLV_REVIEWS' ) && define( 'WPCLV_REVIEWS', 'https://wordpress.org/support/plugin/wpc-linked-variation/reviews/?filter=5' );
! defined( 'WPCLV_CHANGELOG' ) && define( 'WPCLV_CHANGELOG', 'https://wordpress.org/plugins/wpc-linked-variation/#developers' );
! defined( 'WPCLV_DISCUSSION' ) && define( 'WPCLV_DISCUSSION', 'https://wordpress.org/support/plugin/wpc-linked-variation' );
! defined( 'WPC_URI' ) && define( 'WPC_URI', WPCLV_URI );

include 'includes/wpc-dashboard.php';
include 'includes/wpc-menu.php';
include 'includes/wpc-kit.php';

if ( ! function_exists( 'wpclv_init' ) ) {
	add_action( 'plugins_loaded', 'wpclv_init', 11 );

	function wpclv_init() {
		// load text-domain
		load_plugin_textdomain( 'wpc-linked-variation', false, basename( __DIR__ ) . '/languages/' );

		if ( ! function_exists( 'WC' ) || ! version_compare( WC()->version, '3.0', '>=' ) ) {
			add_action( 'admin_notices', 'wpclv_notice_wc' );

			return;
		}

		if ( ! class_exists( 'WPCleverWpclv' ) ) {
			class WPCleverWpclv {
				protected static $settings = array();
				protected static $links = array();
				protected static $instance = null;

				public static function instance() {
					if ( is_null( self::$instance ) ) {
						self::$instance = new self();
					}

					return self::$instance;
				}

				function __construct() {
					self::$links    = (array) get_option( 'wpclv_links', [] );
					self::$settings = (array) get_option( 'wpclv_settings', [] );

					// init
					add_action( 'init', [ $this, 'init' ] );

					// meta box
					add_action( 'add_meta_boxes', [ $this, 'add_meta_boxes' ] );
					add_action( 'save_post_wpclv', [ $this, 'save_meta_boxes' ] );

					// column
					add_filter( 'manage_edit-wpclv_columns', [ $this, 'custom_column' ], 10 );
					add_action( 'manage_wpclv_posts_custom_column', [ $this, 'custom_column_value' ], 10, 2 );

					add_action( 'wp_ajax_wpclv_add_link', [ $this, 'add_link' ] );
					add_action( 'wp_enqueue_scripts', [ $this, 'wp_enqueue_scripts' ] );
					add_action( 'admin_enqueue_scripts', [ $this, 'admin_enqueue_scripts' ] );
					add_action( 'admin_init', [ $this, 'register_settings' ] );
					add_action( 'admin_menu', [ $this, 'admin_menu' ] );

					switch ( self::get_setting( 'position', 'above' ) ) {
						case 'above':
							add_action( 'woocommerce_single_product_summary', [ $this, 'render' ], 25 );
							break;
						case 'below':
							add_action( 'woocommerce_single_product_summary', [ $this, 'render' ], 35 );
							break;
						case 'below_title':
							add_action( 'woocommerce_single_product_summary', [ $this, 'render' ], 6 );
							break;
						case 'below_price':
							add_action( 'woocommerce_single_product_summary', [ $this, 'render' ], 11 );
							break;
						case 'below_excerpt':
							add_action( 'woocommerce_single_product_summary', [ $this, 'render' ], 21 );
							break;
					}

					// link
					add_filter( 'plugin_action_links', [ $this, 'action_links' ], 10, 2 );
					add_filter( 'plugin_row_meta', [ $this, 'row_meta' ], 10, 2 );

					// ajax
					add_action( 'wp_ajax_wpclv_search_term', [ $this, 'search_term' ] );

					// WPC Smart Messages
					add_filter( 'wpcsm_locations', [ $this, 'wpcsm_locations' ] );

					// HPOS compatibility
					add_action( 'before_woocommerce_init', function () {
						if ( class_exists( '\Automattic\WooCommerce\Utilities\FeaturesUtil' ) ) {
							\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', WPCLV_FILE );
						}
					} );
				}

				public static function get_settings() {
					return apply_filters( 'wpclv_get_settings', self::$settings );
				}

				public static function get_setting( $name, $default = false ) {
					if ( ! empty( self::$settings ) && isset( self::$settings[ $name ] ) ) {
						$setting = self::$settings[ $name ];
					} else {
						$setting = get_option( 'wpclv_' . $name, $default );
					}

					return apply_filters( 'wpclv_get_setting', $setting, $name, $default );
				}

				function init() {
					// shortcode
					add_shortcode( 'wpclv', array( $this, 'shortcode' ) );

					// post type
					$labels = array(
						'name'          => _x( 'Linked Variations', 'Post Type General Name', 'wpc-linked-variation' ),
						'singular_name' => _x( 'Linked Variation', 'Post Type Singular Name', 'wpc-linked-variation' ),
						'add_new_item'  => esc_html__( 'Add New Linked Variation', 'wpc-linked-variation' ),
						'add_new'       => esc_html__( 'Add New', 'wpc-linked-variation' ),
						'edit_item'     => esc_html__( 'Edit Linked Variation', 'wpc-linked-variation' ),
						'update_item'   => esc_html__( 'Update Linked Variation', 'wpc-linked-variation' ),
						'search_items'  => esc_html__( 'Search Linked Variation', 'wpc-linked-variation' ),
					);

					$args = array(
						'label'               => esc_html__( 'Linked Variation', 'wpc-linked-variation' ),
						'labels'              => $labels,
						'supports'            => array( 'title' ),
						'hierarchical'        => false,
						'public'              => false,
						'show_ui'             => true,
						'show_in_menu'        => true,
						'show_in_nav_menus'   => true,
						'show_in_admin_bar'   => true,
						'menu_position'       => 28,
						'menu_icon'           => 'dashicons-admin-links',
						'can_export'          => true,
						'has_archive'         => false,
						'exclude_from_search' => true,
						'publicly_queryable'  => false,
						'capability_type'     => 'post',
						'show_in_rest'        => false,
					);

					register_post_type( 'wpclv', $args );
				}

				function shortcode( $attrs ) {
					$output = '';
					$attrs  = shortcode_atts( array( 'id' => null ), $attrs );

					if ( ! $attrs['id'] ) {
						global $product;

						if ( $product ) {
							$attrs['id'] = $product->get_id();
						}
					}

					if ( $attrs['id'] ) {
						ob_start();
						self::render( $attrs['id'] );
						$output = ob_get_clean();
					}

					return $output;
				}

				function add_meta_boxes() {
					add_meta_box( 'wpclv_configuration', esc_html__( 'Configuration', 'wpc-linked-variation' ), array(
						$this,
						'meta_box_callback'
					), 'wpclv', 'advanced', 'low' );
				}

				function meta_box_callback( $post ) {
					$post_id = $post->ID;
					$link    = get_post_meta( $post_id, 'wpclv_link', true );
					?>
                    <table class="form-table">
                        <tr>
                            <td colspan="2">
                                <div class="wpclv_links">
									<?php
									if ( ! empty( $link ) ) {
										self::get_link_2( $link );
									} else {
										self::get_link_2();
									}
									?>
                                </div>
                            </td>
                        </tr>
                    </table>
					<?php
				}

				function save_meta_boxes( $post_id ) {
					if ( isset( $_POST['wpclv_link'] ) ) {
						update_post_meta( $post_id, 'wpclv_link', $_POST['wpclv_link'] );
					}
				}

				function custom_column( $columns ) {
					return array(
						'cb'                  => $columns['cb'],
						'title'               => esc_html__( 'Title', 'wpc-linked-variation' ),
						'wpclv_configuration' => esc_html__( 'Configuration', 'wpc-linked-variation' ),
						'date'                => esc_html__( 'Date', 'wpc-linked-variation' )

					);
				}

				function custom_column_value( $column, $postid ) {
					if ( $column == 'wpclv_configuration' ) {
						$info = get_post_meta( $postid, 'wpclv_link', true );

						if ( is_array( $info ) && ! empty( $info ) ) {
							if ( isset( $info['source'] ) ) {
								// source
								switch ( $info['source'] ) {
									case 'products':
										echo esc_html__( 'Products', 'wpc-linked-variation' ) . ': ';
										$names = [];

										if ( isset( $info['products'] ) && ! empty( $info['products'] ) ) {
											$products = explode( ',', $info['products'] );

											foreach ( $products as $pid ) {
												if ( $name = get_the_title( $pid ) ) {
													$names[] = $name;
												}
											}

											echo implode( ', ', $names );
										}

										break;
									case 'categories':
										echo esc_html__( 'Categories', 'wpc-linked-variation' ) . ': ' . $info['categories'];

										break;
									case 'tags':
										echo esc_html__( 'Tags', 'wpc-linked-variation' ) . ': ' . $info['tags'];

										break;
									default:
										if ( $taxonomy = get_taxonomy( $info['source'] ) ) {
											echo esc_html( $taxonomy->label ) . ': ' . $info['terms'];
										}

										break;
								}
							}

							if ( isset( $info['attributes'] ) && ! empty( $info['attributes'] ) ) {
								// attributes
								echo '<br/>';
								echo esc_html__( 'Attributes', 'wpc-linked-variation' ) . ': ';
								$attr_names = [];

								foreach ( $info['attributes'] as $attr_id ) {
									if ( $attr = wc_get_attribute( absint( str_replace( 'id:', '', $attr_id ) ) ) ) {
										$attr_names[] = $attr->name;
									}
								}

								if ( ! empty( $attr_names ) ) {
									echo implode( ', ', $attr_names );
								}
							}
						}
					}
				}

				function wp_enqueue_scripts() {
					if ( self::get_setting( 'tooltip', 'top' ) !== 'no' ) {
						wp_enqueue_style( 'hint', WPCLV_URI . 'assets/css/hint.css' );
					}

					wp_enqueue_style( 'wpclv-frontend', WPCLV_URI . 'assets/css/frontend.css', array(), WPCLV_VERSION );
				}

				function admin_enqueue_scripts() {
					wp_enqueue_style( 'wpclv-backend', WPCLV_URI . 'assets/css/backend.css', array( 'woocommerce_admin_styles' ), WPCLV_VERSION );
					wp_enqueue_script( 'wpclv-backend', WPCLV_URI . 'assets/js/backend.js', array(
						'jquery',
						'wc-enhanced-select',
						'jquery-ui-sortable',
						'selectWoo'
					), WPCLV_VERSION, true );
					wp_localize_script( 'wpclv-backend', 'wpclv_vars', array(
							'wpclv_nonce' => wp_create_nonce( 'wpclv_nonce' )
						)
					);
				}

				function register_settings() {
					// settings
					register_setting( 'wpclv_settings', 'wpclv_settings' );
				}

				function admin_menu() {
					add_submenu_page( 'wpclever', esc_html__( 'WPC Linked Variation', 'wpc-linked-variation' ), esc_html__( 'Linked Variation', 'wpc-linked-variation' ), 'manage_options', 'wpclever-wpclv', array(
						$this,
						'admin_menu_content'
					) );
				}

				function admin_menu_content() {
					add_thickbox();
					$active_tab = isset( $_GET['tab'] ) ? sanitize_key( $_GET['tab'] ) : 'settings';
					?>
                    <div class="wpclever_settings_page wrap">
                        <h1 class="wpclever_settings_page_title"><?php echo esc_html__( 'WPC Linked Variation', 'wpc-linked-variation' ) . ' ' . WPCLV_VERSION; ?></h1>
                        <div class="wpclever_settings_page_desc about-text">
                            <p>
								<?php printf( esc_html__( 'Thank you for using our plugin! If you are satisfied, please reward it a full five-star %s rating.', 'wpc-linked-variation' ), '<span style="color:#ffb900">&#9733;&#9733;&#9733;&#9733;&#9733;</span>' ); ?>
                                <br/>
                                <a href="<?php echo esc_url( WPCLV_REVIEWS ); ?>"
                                   target="_blank"><?php esc_html_e( 'Reviews', 'wpc-linked-variation' ); ?></a> | <a
                                        href="<?php echo esc_url( WPCLV_CHANGELOG ); ?>"
                                        target="_blank"><?php esc_html_e( 'Changelog', 'wpc-linked-variation' ); ?></a>
                                | <a href="<?php echo esc_url( WPCLV_DISCUSSION ); ?>"
                                     target="_blank"><?php esc_html_e( 'Discussion', 'wpc-linked-variation' ); ?></a>
                            </p>
                        </div>
						<?php if ( isset( $_GET['settings-updated'] ) && $_GET['settings-updated'] ) { ?>
                            <div class="notice notice-success is-dismissible">
                                <p><?php esc_html_e( 'Settings updated.', 'wpc-linked-variation' ); ?></p>
                            </div>
						<?php } ?>
                        <div class="wpclever_settings_page_nav">
                            <h2 class="nav-tab-wrapper">
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-wpclv&tab=settings' ); ?>"
                                   class="<?php echo $active_tab === 'settings' ? 'nav-tab nav-tab-active' : 'nav-tab'; ?>">
									<?php esc_html_e( 'Settings', 'wpc-linked-variation' ); ?>
                                </a>
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-wpclv&tab=linked' ); ?>"
                                   class="<?php echo $active_tab === 'linked' ? 'nav-tab nav-tab-active' : 'nav-tab'; ?>">
									<?php esc_html_e( 'Linked Variations', 'wpc-linked-variation' ); ?>
                                </a>
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-wpclv&tab=premium' ); ?>"
                                   class="<?php echo $active_tab === 'premium' ? 'nav-tab nav-tab-active' : 'nav-tab'; ?>"
                                   style="color: #c9356e">
									<?php esc_html_e( 'Premium Version', 'wpc-linked-variation' ); ?>
                                </a>
                                <a href="<?php echo esc_url( admin_url( 'admin.php?page=wpclever-kit' ) ); ?>"
                                   class="nav-tab">
									<?php esc_html_e( 'Essential Kit', 'wpc-linked-variation' ); ?>
                                </a>
                            </h2>
                        </div>
                        <div class="wpclever_settings_page_content">
							<?php if ( $active_tab === 'settings' ) {
								$position              = self::get_setting( 'position', 'above' );
								$tooltip               = self::get_setting( 'tooltip', 'top' );
								$hide_empty            = self::get_setting( 'hide_empty', 'yes' );
								$exclude_hidden        = self::get_setting( 'exclude_hidden', 'no' );
								$exclude_unpurchasable = self::get_setting( 'exclude_unpurchasable', 'no' );
								$link                  = self::get_setting( 'link', 'yes' );
								$nofollow              = self::get_setting( 'nofollow', 'no' );
								?>
                                <form method="post" action="options.php">
                                    <table class="form-table">
                                        <tr>
                                            <th><?php esc_html_e( 'Position', 'wpc-linked-variation' ); ?></th>
                                            <td>
                                                <select name="wpclv_settings[position]">
                                                    <option value="above" <?php selected( $position, 'above' ); ?>><?php esc_html_e( 'Above the add to cart button', 'wpc-linked-variation' ); ?></option>
                                                    <option value="below" <?php selected( $position, 'below' ); ?>><?php esc_html_e( 'Under the add to cart button', 'wpc-linked-variation' ); ?></option>
                                                    <option value="below_title" <?php selected( $position, 'below_title' ); ?>><?php esc_html_e( 'Under the title', 'wpc-linked-variation' ); ?></option>
                                                    <option value="below_price" <?php selected( $position, 'below_price' ); ?>><?php esc_html_e( 'Under the price', 'wpc-linked-variation' ); ?></option>
                                                    <option value="below_excerpt" <?php selected( $position, 'below_excerpt' ); ?>><?php esc_html_e( 'Under the excerpt', 'wpc-linked-variation' ); ?></option>
                                                    <option value="no" <?php selected( $position, 'no' ); ?>><?php esc_html_e( 'No (hide it)', 'wpc-linked-variation' ); ?></option>
                                                </select>
                                                <span class="description"><?php esc_html_e( 'Choose the position to show the linked variations.', 'wpc-linked-variation' ); ?></span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Shortcode', 'wpc-linked-variation' ); ?></th>
                                            <td>
												<?php printf( esc_html__( 'You can use the shortcode %s to show the list where you want.', 'wpc-linked-variation' ), '<code>[wpclv]</code>' ); ?>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Tooltip position', 'wpc-linked-variation' ); ?></th>
                                            <td>
                                                <select name="wpclv_settings[tooltip]">
                                                    <option value="top" <?php selected( $tooltip, 'top' ); ?>><?php esc_html_e( 'Top', 'wpc-linked-variation' ); ?></option>
                                                    <option value="right" <?php selected( $tooltip, 'right' ); ?>><?php esc_html_e( 'Right', 'wpc-linked-variation' ); ?></option>
                                                    <option value="bottom" <?php selected( $tooltip, 'bottom' ); ?>><?php esc_html_e( 'Bottom', 'wpc-linked-variation' ); ?></option>
                                                    <option value="left" <?php selected( $tooltip, 'left' ); ?>><?php esc_html_e( 'Left', 'wpc-linked-variation' ); ?></option>
                                                    <option value="no" <?php selected( $tooltip, 'no' ); ?>><?php esc_html_e( 'No (hide it)', 'wpc-linked-variation' ); ?></option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Hide empty attribute terms', 'wpc-linked-variation' ); ?></th>
                                            <td>
                                                <select name="wpclv_settings[hide_empty]">
                                                    <option value="yes" <?php selected( $hide_empty, 'yes' ); ?>><?php esc_html_e( 'Yes', 'wpc-linked-variation' ); ?></option>
                                                    <option value="no" <?php selected( $hide_empty, 'no' ); ?>><?php esc_html_e( 'No', 'wpc-linked-variation' ); ?></option>
                                                </select>
                                                <span class="description"><?php esc_html_e( 'Hide attribute terms that haven\'t any products.', 'wpc-linked-variation' ); ?></span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Exclude hidden product', 'wpc-linked-variation' ); ?></th>
                                            <td>
                                                <select name="wpclv_settings[exclude_hidden]">
                                                    <option value="yes" <?php selected( $exclude_hidden, 'yes' ); ?>><?php esc_html_e( 'Yes', 'wpc-linked-variation' ); ?></option>
                                                    <option value="no" <?php selected( $exclude_hidden, 'no' ); ?>><?php esc_html_e( 'No', 'wpc-linked-variation' ); ?></option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Exclude unpurchasable product', 'wpc-linked-variation' ); ?></th>
                                            <td>
                                                <select name="wpclv_settings[exclude_unpurchasable]">
                                                    <option value="yes" <?php selected( $exclude_unpurchasable, 'yes' ); ?>><?php esc_html_e( 'Yes', 'wpc-linked-variation' ); ?></option>
                                                    <option value="no" <?php selected( $exclude_unpurchasable, 'no' ); ?>><?php esc_html_e( 'No', 'wpc-linked-variation' ); ?></option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Link to individual product', 'wpc-linked-variation' ); ?></th>
                                            <td>
                                                <select name="wpclv_settings[link]">
                                                    <option value="yes" <?php selected( $link, 'yes' ); ?>><?php esc_html_e( 'Open in the same tab', 'wpc-linked-variation' ); ?></option>
                                                    <option value="yes_blank" <?php selected( $link, 'yes_blank' ); ?>><?php esc_html_e( 'Open in the new tab', 'wpc-linked-variation' ); ?></option>
                                                    <option value="yes_popup" <?php selected( $link, 'yes_popup' ); ?>><?php esc_html_e( 'Open quick view popup', 'wpc-linked-variation' ); ?></option>
                                                </select>
                                                <span class="description">If you choose "Open quick view popup", please install <a
                                                            href="<?php echo esc_url( admin_url( 'plugin-install.php?tab=plugin-information&plugin=woo-smart-quick-view&TB_iframe=true&width=800&height=550' ) ); ?>"
                                                            class="thickbox" title="Install WPC Smart Quick View">WPC Smart Quick View</a> to make it work.</span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Use nofollow links', 'wpc-linked-variation' ); ?></th>
                                            <td>
                                                <select name="wpclv_settings[nofollow]">
                                                    <option value="yes" <?php selected( $nofollow, 'yes' ); ?>><?php esc_html_e( 'Yes', 'wpc-linked-variation' ); ?></option>
                                                    <option value="no" <?php selected( $nofollow, 'no' ); ?>><?php esc_html_e( 'No', 'wpc-linked-variation' ); ?></option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr class="heading">
                                            <th colspan="2"><?php esc_html_e( 'Suggestion', 'wpc-linked-variation' ); ?></th>
                                        </tr>
                                        <tr>
                                            <td colspan="2">
                                                To display custom engaging real-time messages on any wished positions,
                                                please install <a
                                                        href="https://wordpress.org/plugins/wpc-smart-messages/"
                                                        target="_blank">WPC Smart Messages for WooCommerce</a> plugin.
                                                It's free and
                                                available now on the WordPress repository.
                                            </td>
                                        </tr>
                                        <tr class="submit">
                                            <th colspan="2">
												<?php settings_fields( 'wpclv_settings' ); ?>
												<?php submit_button(); ?>
                                            </th>
                                        </tr>
                                    </table>
                                </form>
							<?php } elseif ( $active_tab === 'linked' ) {
								if ( isset( $_GET['action'] ) && ( $_GET['action'] === 'convert' ) ) {
									if ( is_array( self::$links ) && ( count( self::$links ) > 0 ) ) {
										foreach ( array_reverse( self::$links ) as $link ) {
											$new_link = array(
												'post_type'   => 'wpclv',
												'post_title'  => uniqid( 'wpclv-' ),
												'post_status' => 'publish',
											);

											$new_link_id = wp_insert_post( $new_link );

											if ( $new_link_id ) {
												update_post_meta( $new_link_id, 'wpclv_link', $link );
											}
										}

										// delete old data
										delete_option( 'wpclv_links' );
									}

									wp_safe_redirect( admin_url( 'edit.php?post_type=wpclv' ) );
								}
								?>
                                <form method="post" action="options.php">
									<?php if ( ! empty( self::$links ) ) { ?>
                                        <table class="form-table">
                                            <tr>
                                                <td colspan="2">
                                                    <div style="text-align: center; color: #c9356e">
                                                        From version 3.0, we used a new data structure for Linked
                                                        Variations.
                                                        <a href="<?php echo admin_url( 'admin.php?page=wpclever-wpclv&tab=linked&action=convert' ); ?>">Click
                                                            here</a> to convert your data then you can manage
                                                        linked variations at
                                                        the new menu on the left panel.<br/>
                                                        If you already added too many linked variations here, please
                                                        backup your website data first for safety.
                                                    </div>
                                                </td>
                                            </tr>
                                        </table>
                                        <table class="form-table" style="opacity: .5; pointer-events: none">
                                            <tr>
                                                <td colspan="2">
                                                    <div class="wpclv_links">
														<?php
														if ( is_array( self::$links ) && ( count( self::$links ) > 0 ) ) {
															foreach ( self::$links as $link ) {
																self::get_link( $link );
															}
														}
														?>
                                                    </div>
                                                </td>
                                            </tr>
                                        </table>
									<?php } else { ?>
                                        <table class="form-table">
                                            <tr>
                                                <td colspan="2">
                                                    <div style="text-align: center">
                                                        From version 3.0, we used a new data structure for Linked
                                                        Variations. You can manage linked variations at the <a
                                                                href="<?php echo admin_url( 'edit.php?post_type=wpclv' ); ?>">new
                                                            menu</a> on the
                                                        left panel.
                                                    </div>
                                                </td>
                                            </tr>
                                        </table>
									<?php } ?>
                                </form>
							<?php } elseif ( $active_tab == 'premium' ) { ?>
                                <div class="wpclever_settings_page_content_text">
                                    <p>
                                        Get the Premium Version just $29! <a
                                                href="https://wpclever.net/downloads/wpc-linked-variation?utm_source=pro&utm_medium=wooco&utm_campaign=wporg"
                                                target="_blank">https://wpclever.net/downloads/wpc-linked-variation</a>
                                    </p>
                                    <p><strong>Extra features for Premium Version:</strong></p>
                                    <ul style="margin-bottom: 0">
                                        <li>- Use Categories, Tags, or Attributes as the source.</li>
                                        <li>- Get the lifetime update & premium support.</li>
                                    </ul>
                                </div>
							<?php } ?>
                        </div>
                    </div>
					<?php
				}

				function add_link() {
					if ( ! isset( $_POST['wpclv_nonce'] ) || ! wp_verify_nonce( $_POST['wpclv_nonce'], 'wpclv_nonce' ) ) {
						die( 'Permissions check failed!' );
					}

					self::get_link();
					wp_die();
				}

				function get_link( $link = null ) {
					// old version
					isset( $link['attributes'] ) ?: $link['attributes'] = array();
					isset( $link['images'] ) ?: $link['images'] = array();
					isset( $link['swatches'] ) ?: $link['swatches'] = array();

					$link_key        = uniqid( 'wpclv-' );
					$link_source     = isset( $link['source'] ) ? $link['source'] : 'products';
					$link_products   = isset( $link['products'] ) ? $link['products'] : '';
					$link_categories = isset( $link['categories'] ) ? $link['categories'] : '';
					$link_tags       = isset( $link['tags'] ) ? $link['tags'] : '';
					$wc_attributes   = wc_get_attribute_taxonomies();
					$attributes      = array();

					foreach ( $wc_attributes as $wc_attribute ) {
						$attributes[ 'id:' . $wc_attribute->attribute_id ] = $wc_attribute->attribute_label;
					}
					?>
                    <div class="wpclv_link">
                        <div class="wpclv_tr wpclv_tr_heading">
                            <span class="wpclv_link_move"><?php echo '#' . $link_key; ?></span>
                            <span class="wpclv_link_remove">&times;</span>
                        </div>
                        <div class="wpclv_tr">
                            <div class="wpclv_th">
                                <select class="wpclv-source" name="wpclv_links[<?php echo $link_key; ?>][source]">
                                    <option value="products" <?php selected( $link_source, 'products' ); ?>><?php esc_html_e( 'Products', 'wpc-linked-variation' ); ?></option>
                                    <option value="categories" <?php selected( $link_source, 'categories' ); ?>
                                            disabled><?php esc_html_e( 'Categories', 'wpc-linked-variation' ); ?></option>
                                    <option value="tags" <?php selected( $link_source, 'tags' ); ?>
                                            disabled><?php esc_html_e( 'Tags', 'wpc-linked-variation' ); ?></option>
                                </select>
                            </div>
                            <div class="wpclv_td wpclv_link_td">
                                <div class="wpclv-source-hide wpclv-source-products">
                                    <input class="wpclv-products" name="wpclv_links[<?php echo $link_key; ?>][products]"
                                           type="hidden" value="<?php echo esc_attr( $link_products ); ?>"/>
                                    <select class="wc-product-search wpclv-product-search" multiple="multiple"
                                            data-placeholder="<?php esc_attr_e( 'Search for a product&hellip;', 'wpc-linked-variation' ); ?>"
                                            data-action="woocommerce_json_search_products">
										<?php
										$_product_ids = explode( ',', $link_products );

										foreach ( $_product_ids as $_product_id ) {
											$_product = wc_get_product( $_product_id );

											if ( $_product ) {
												echo '<option value="' . esc_attr( $_product_id ) . '" selected disabled>' . wp_kses_post( $_product->get_formatted_name() ) . '</option>';
											}
										}
										?>
                                    </select>
									<?php echo '<script>jQuery(document.body).trigger( \'wc-enhanced-select-init\' );</script>'; ?>
                                </div>
                                <div class="wpclv-source-hide wpclv-source-categories">
                                    <input class="wpclv-categories" type="hidden"
                                           name="wpclv_links[<?php echo $link_key; ?>][categories]"
                                           value="<?php echo esc_attr( $link_categories ); ?>"/>
                                    <select class="wc-category-search wpclv-category-search" multiple="multiple"
                                            data-placeholder="<?php esc_attr_e( 'Search for a category&hellip;', 'wpc-linked-variation' ); ?>">
										<?php
										$category_slugs = explode( ',', $link_categories );

										if ( count( $category_slugs ) > 0 ) {
											foreach ( $category_slugs as $category_slug ) {
												$category = get_term_by( 'slug', $category_slug, 'product_cat' );

												if ( $category ) {
													echo '<option value="' . esc_attr( $category_slug ) . '" selected>' . wp_kses_post( $category->name ) . '</option>';
												}
											}
										}
										?>
                                    </select>
									<?php echo '<script>jQuery(document.body).trigger( \'wc-enhanced-select-init\' );</script>'; ?>
                                </div>
                                <div class="wpclv-source-hide wpclv-source-tags">
                                    <input class="wpclv-tags" type="text" style="width: 100%"
                                           name="wpclv_links[<?php echo $link_key; ?>][tags]"
                                           value="<?php echo esc_attr( $link_tags ); ?>"/>
                                    <span class="description"><?php esc_html_e( 'Add some tags, split by a comma.', 'wpc-linked-variation' ); ?></span>
                                </div>
                            </div>
                        </div>
                        <div class="wpclv_tr">
                            <div class="wpclv_th"><?php esc_html_e( 'Linked by (attributes)', 'wpc-linked-variation' ); ?></div>
                            <div class="wpclv_td wpclv_link_td">
								<?php
								$saved_attributes = $merge_attributes = array();

								foreach ( $link['attributes'] as $attr ) {
									$saved_attributes[ $attr ] = $attributes[ $attr ];
								}

								$merge_attributes = array_merge( $saved_attributes, $attributes );

								if ( $merge_attributes ) {
									echo '<div class="wpclv-attributes">';

									foreach ( $merge_attributes as $attribute_id => $attribute_label ) {
										echo '<div class="wpclv-attribute"><span class="move">' . esc_html__( 'Move', 'wpc-linked-variation' ) . '</span><span class="checkbox"><label><input type="checkbox" name="wpclv_links[' . $link_key . '][attributes][]" value="' . $attribute_id . '" ' . ( is_array( $link['attributes'] ) && in_array( $attribute_id, $link['attributes'] ) ? 'checked' : '' ) . '/>' . $attribute_label . '</label></span><span class="display"><label><input type="checkbox" name="wpclv_links[' . $link_key . '][images][]" value="' . $attribute_id . '" ' . ( is_array( $link['images'] ) && in_array( $attribute_id, $link['images'] ) ? 'checked' : '' ) . '/>' . esc_html__( 'Show images', 'wpc-linked-variation' ) . '</label></span></div>';
									}

									echo '</div>';
								}
								?>
                            </div>
                        </div>
                    </div>
					<?php
				}

				function get_link_2( $link = null ) {
					isset( $link['attributes'] ) ?: $link['attributes'] = array();
					isset( $link['images'] ) ?: $link['images'] = array();
					isset( $link['swatches'] ) ?: $link['swatches'] = array();

					$link_source     = isset( $link['source'] ) ? $link['source'] : 'products';
					$link_products   = isset( $link['products'] ) ? $link['products'] : '';
					$link_categories = isset( $link['categories'] ) ? $link['categories'] : '';
					$link_tags       = isset( $link['tags'] ) ? $link['tags'] : '';
					$link_terms      = isset( $link['terms'] ) ? $link['terms'] : '';
					$wc_attributes   = wc_get_attribute_taxonomies();
					$attributes      = array();

					foreach ( $wc_attributes as $wc_attribute ) {
						$attributes[ 'id:' . $wc_attribute->attribute_id ] = $wc_attribute->attribute_label;
					}
					?>
                    <div class="wpclv_link">
                        <div class="wpclv_tr">
                            <div class="wpclv_th"><?php esc_html_e( 'Source', 'wpc-linked-variation' ); ?></div>
                            <div class="wpclv_td">
                                <select class="wpclv-source" name="wpclv_link[source]">
                                    <option value="products" <?php selected( $link_source, 'products' ); ?>><?php esc_html_e( 'Products', 'wpc-linked-variation' ); ?></option>
                                    <option value="categories" <?php selected( $link_source, 'categories' ); ?>
                                            disabled><?php esc_html_e( 'Categories (deprecated)', 'wpc-linked-variation' ); ?></option>
                                    <option value="tags" <?php selected( $link_source, 'tags' ); ?>
                                            disabled><?php esc_html_e( 'Tags (deprecated)', 'wpc-linked-variation' ); ?></option>
									<?php
									$taxonomies = get_object_taxonomies( 'product', 'objects' ); //$taxonomies = get_taxonomies( [ 'object_type' => [ 'product' ] ], 'objects' );

									foreach ( $taxonomies as $taxonomy ) {
										echo '<option value="' . $taxonomy->name . '" ' . ( $link_source === $taxonomy->name ? 'selected' : '' ) . ' disabled>' . $taxonomy->label . '</option>';
									}
									?>
                                </select>
                            </div>
                        </div>
                        <div class="wpclv_tr wpclv-source-hide wpclv-source-products">
                            <div class="wpclv_th">
								<?php esc_html_e( 'Products', 'wpc-linked-variation' ); ?>
                            </div>
                            <div class="wpclv_td wpclv_link_td">
                                <input class="wpclv-products" type="hidden" name="wpclv_link[products]"
                                       value="<?php echo esc_attr( $link_products ); ?>"/>
                                <select class="wc-product-search wpclv-product-search" multiple="multiple"
                                        data-placeholder="<?php esc_attr_e( 'Search for a product&hellip;', 'wpc-linked-variation' ); ?>"
                                        data-action="woocommerce_json_search_products">
									<?php
									$_product_ids = explode( ',', $link_products );

									foreach ( $_product_ids as $_product_id ) {
										$_product = wc_get_product( $_product_id );

										if ( $_product ) {
											echo '<option value="' . esc_attr( $_product_id ) . '" selected>' . wp_kses_post( $_product->get_formatted_name() ) . '</option>';
										}
									}
									?>
                                </select>
								<?php echo '<script>jQuery(document.body).trigger( \'wc-enhanced-select-init\' );</script>'; ?>
                            </div>
                        </div>
                        <div class="wpclv_tr wpclv-source-hide wpclv-source-categories">
                            <div class="wpclv_th">
								<?php esc_html_e( 'Categories', 'wpc-linked-variation' ); ?>
                            </div>
                            <div class="wpclv_td wpclv_link_td">
                                <input class="wpclv-categories" type="hidden" name="wpclv_link[categories]"
                                       value="<?php echo esc_attr( $link_categories ); ?>"/>
                                <select class="wc-category-search wpclv-category-search" multiple="multiple"
                                        data-placeholder="<?php esc_attr_e( 'Search for a category&hellip;', 'wpc-linked-variation' ); ?>">
									<?php
									$category_slugs = explode( ',', $link_categories );

									if ( count( $category_slugs ) > 0 ) {
										foreach ( $category_slugs as $category_slug ) {
											$category = get_term_by( 'slug', $category_slug, 'product_cat' );

											if ( $category ) {
												echo '<option value="' . esc_attr( $category_slug ) . '" selected>' . wp_kses_post( $category->name ) . '</option>';
											}
										}
									}
									?>
                                </select>
								<?php echo '<script>jQuery(document.body).trigger( \'wc-enhanced-select-init\' );</script>'; ?>
                            </div>
                        </div>
                        <div class="wpclv_tr wpclv-source-hide wpclv-source-tags">
                            <div class="wpclv_th">
								<?php esc_html_e( 'Tags', 'wpc-linked-variation' ); ?>
                            </div>
                            <div class="wpclv_td wpclv_link_td">
                                <input class="wpclv-tags" name="wpclv_link[tags]" type="text" style="width: 100%"
                                       value="<?php echo esc_attr( $link_tags ); ?>"/>
                                <span class="description"><?php esc_html_e( 'Add some tags, split by a comma.', 'wpc-linked-variation' ); ?></span>
                            </div>
                        </div>
                        <div class="wpclv_tr">
                            <div class="wpclv_th"><?php esc_html_e( 'Linked by (attributes)', 'wpc-linked-variation' ); ?></div>
                            <div class="wpclv_td wpclv_link_td">
								<?php
								$saved_attributes = $merge_attributes = array();

								foreach ( $link['attributes'] as $attr ) {
									$saved_attributes[ $attr ] = $attributes[ $attr ];
								}

								$merge_attributes = array_merge( $saved_attributes, $attributes );

								if ( $merge_attributes ) {
									echo '<div class="wpclv-attributes">';

									foreach ( $merge_attributes as $attribute_id => $attribute_label ) {
										echo '<div class="wpclv-attribute"><span class="move">' . esc_html__( 'Move', 'wpc-linked-variation' ) . '</span><span class="checkbox"><label><input type="checkbox" name="wpclv_link[attributes][]" value="' . $attribute_id . '" ' . ( is_array( $link['attributes'] ) && in_array( $attribute_id, $link['attributes'] ) ? 'checked' : '' ) . '/>' . $attribute_label . '</label></span><span class="display"><label><input type="checkbox" class="wpclv_display_checkbox" name="wpclv_link[images][]" value="' . $attribute_id . '" ' . ( is_array( $link['images'] ) && in_array( $attribute_id, $link['images'] ) ? 'checked' : '' ) . '/>' . esc_html__( 'Show images', 'wpc-linked-variation' ) . '</label></span><span class="display"><label><input type="checkbox" class="wpclv_display_checkbox" name="wpclv_link[swatches][]" value="' . $attribute_id . '" ' . ( isset( $link['swatches'] ) && is_array( $link['swatches'] ) && in_array( $attribute_id, $link['swatches'] ) ? 'checked' : '' ) . '/>' . esc_html__( 'Use swatches', 'wpc-linked-variation' ) . '</label></span></div>';
									}

									echo '</div>';
								}
								?>
                            </div>
                        </div>
                        <div class="wpclv_tr">
                            <div class="wpclv_th"></div>
                            <div class="wpclv_td" style="text-align: end;">
								<?php add_thickbox(); ?>
                                To use swatches, you need to install and activate <a
                                        href="<?php echo esc_url( admin_url( 'plugin-install.php?tab=plugin-information&plugin=wpc-variation-swatches&TB_iframe=true&width=800&height=550' ) ); ?>"
                                        class="thickbox" title="Install WPC Variation Swatches">WPC Variation
                                    Swatches</a>.
                            </div>
                        </div>
                    </div>
					<?php
				}

				public static function term( $type, $attribute, $term, $active, $product_id = 0 ) {
					$link       = self::get_setting( 'link', 'yes' );
					$nofollow   = self::get_setting( 'nofollow', 'no' ) === 'yes';
					$hint       = self::get_setting( 'tooltip', 'top' );
					$hint_class = $hint !== 'no' ? 'hint--' . $hint : '';

					switch ( $type ) {
						case 'swatches':
							$attribute_type = isset( $attribute->type ) ? $attribute->type : 'select';

							if ( ! in_array( $attribute_type, array( 'button', 'color', 'image' ) ) ) {
								$attribute_type = 'button';
							}

							switch ( $attribute_type ) {
								case 'button' :
									$val     = get_term_meta( $term->term_id, 'wpcvs_button', true ) ?: $term->name;
									$tooltip = get_term_meta( $term->term_id, 'wpcvs_tooltip', true ) ?: $val;
									$html    = '<div class="wpclv-term ' . $hint_class . ' ' . ( $active ? 'active' : '' ) . '" aria-label="' . esc_attr( $tooltip ) . '">';

									if ( $product_id && ! $active ) {
										$html .= '<a href="' . ( $link === 'yes_popup' ? 'javascript:void(0);' : get_the_permalink( $product_id ) ) . '" ' . ( $nofollow ? 'rel="nofollow"' : '' ) . ' title="' . esc_attr( apply_filters( 'wpclv_term_title', get_the_title( $product_id ), $term, $product_id ) ) . '" ' . ( $link === 'yes_popup' ? 'class="woosq-link" data-id="' . $product_id . '"' : '' ) . ' ' . ( $link === 'yes_blank' ? 'target="_blank"' : '' ) . '>' . esc_html( $val ) . '</a>';
									} else {
										$html .= '<span>' . esc_html( $val ) . '</span>';
									}

									$html .= '</div>';

									break;
								case 'color':
									$val     = get_term_meta( $term->term_id, 'wpcvs_color', true ) ?: '';
									$tooltip = get_term_meta( $term->term_id, 'wpcvs_tooltip', true ) ?: $term->name;

									$html = '<div class="wpclv-term wpclv-term-color ' . $hint_class . ' ' . ( $active ? 'active' : '' ) . '" aria-label="' . esc_attr( $tooltip ) . '">';

									if ( $product_id && ! $active ) {
										$html .= '<a ' . ( ! empty( $val ) ? 'style="background-color: ' . esc_attr( $val ) . '"' : '' ) . ' href="' . ( $link === 'yes_popup' ? 'javascript:void(0);' : get_the_permalink( $product_id ) ) . '" ' . ( $nofollow ? 'rel="nofollow"' : '' ) . ' title="' . esc_attr( apply_filters( 'wpclv_term_title', get_the_title( $product_id ), $term, $product_id ) ) . '" ' . ( $link === 'yes_popup' ? 'class="woosq-link" data-id="' . $product_id . '"' : '' ) . ' ' . ( $link === 'yes_blank' ? 'target="_blank"' : '' ) . '>' . esc_html( $val ) . '</a>';
									} else {
										$html .= '<span ' . ( ! empty( $val ) ? 'style="background-color: ' . esc_attr( $val ) . '"' : '' ) . '>' . esc_html( $val ) . '</span>';
									}

									$html .= '</div>';

									break;
								case 'image':
									$val     = get_term_meta( $term->term_id, 'wpcvs_image', true ) ? wp_get_attachment_thumb_url( get_term_meta( $term->term_id, 'wpcvs_image', true ) ) : wc_placeholder_img_src();
									$tooltip = get_term_meta( $term->term_id, 'wpcvs_tooltip', true ) ?: $term->name;

									$html = '<div class="wpclv-term wpclv-term-image ' . $hint_class . ' ' . ( $active ? 'active' : '' ) . '" aria-label="' . esc_attr( $tooltip ) . '">';

									if ( $product_id && ! $active ) {
										$html .= '<a href="' . ( $link === 'yes_popup' ? 'javascript:void(0);' : get_the_permalink( $product_id ) ) . '" ' . ( $nofollow ? 'rel="nofollow"' : '' ) . ' title="' . esc_attr( apply_filters( 'wpclv_term_title', get_the_title( $product_id ), $term, $product_id ) ) . '" ' . ( $link === 'yes_popup' ? 'class="woosq-link" data-id="' . $product_id . '"' : '' ) . ' ' . ( $link === 'yes_blank' ? 'target="_blank"' : '' ) . '><img src="' . esc_url( $val ) . '" alt="' . esc_attr( $term->name ) . '"/></a>';
									} else {
										$html .= '<span><img src="' . esc_url( $val ) . '" alt="' . esc_attr( $term->name ) . '"/></span>';
									}

									$html .= '</div>';

									break;
								default:
									$html = '';
							}

							echo apply_filters( 'wpclv_term_swatches', $html, $term, $product_id );
							break;

						case 'image':
							if ( $product_id && ( $product_thumbnail_id = get_post_thumbnail_id( $product_id ) ) ) {
								$term_image = '<img src="' . wp_get_attachment_image_url( $product_thumbnail_id ) . '" alt="' . esc_attr( $term->name ) . '"/>';
							} else {
								$term_image = wc_placeholder_img();
							}

							$html = '<div class="wpclv-term wpclv-term-image ' . $hint_class . ' ' . ( $active ? 'active' : '' ) . '" aria-label="' . esc_attr( apply_filters( 'wpclv_term_label', $term->name, $term, $product_id ) ) . '">';

							if ( $product_id && ! $active ) {
								$html .= '<a href="' . ( $link === 'yes_popup' ? 'javascript:void(0);' : get_the_permalink( $product_id ) ) . '" ' . ( $nofollow ? 'rel="nofollow"' : '' ) . ' title="' . esc_attr( apply_filters( 'wpclv_term_title', get_the_title( $product_id ), $term, $product_id ) ) . '" ' . ( $link === 'yes_popup' ? 'class="woosq-link" data-id="' . $product_id . '"' : '' ) . ' ' . ( $link === 'yes_blank' ? 'target="_blank"' : '' ) . '>' . $term_image . '</a>';
							} else {
								$html .= '<span>' . $term_image . '</span>';
							}

							$html .= '</div>';

							echo apply_filters( 'wpclv_term_image', $html, $term, $product_id );
							break;

						case 'button':
							$html = '<div class="wpclv-term ' . $hint_class . ' ' . ( $active ? 'active' : '' ) . '" aria-label="' . esc_attr( apply_filters( 'wpclv_term_label', $term->name, $term, $product_id ) ) . '">';

							if ( $product_id && ! $active ) {
								$html .= '<a href="' . ( $link === 'yes_popup' ? 'javascript:void(0);' : get_the_permalink( $product_id ) ) . '" ' . ( $nofollow ? 'rel="nofollow"' : '' ) . ' title="' . esc_attr( apply_filters( 'wpclv_term_title', get_the_title( $product_id ), $term, $product_id ) ) . '" ' . ( $link === 'yes_popup' ? 'class="woosq-link" data-id="' . $product_id . '"' : '' ) . ' ' . ( $link === 'yes_blank' ? 'target="_blank"' : '' ) . '>' . esc_html( $term->name ) . '</a>';
							} else {
								$html .= '<span>' . esc_html( $term->name ) . '</span>';
							}

							$html .= '</div>';

							echo apply_filters( 'wpclv_term_button', $html, $term, $product_id );
							break;
					}
				}

				public static function render( $product_id = null ) {
					if ( ! $product_id ) {
						global $product;
						$_product   = $product;
						$product_id = $_product->get_id();
					} else {
						$_product = wc_get_product( $product_id );
					}

					if ( ! $_product ) {
						return;
					}

					$link_data = self::get_linked_data( $product_id );

					if ( empty( $link_data ) ) {
						return;
					}

					$link_attributes = isset( $link_data['attributes'] ) ? $link_data['attributes'] : array();
					$link_images     = isset( $link_data['images'] ) ? $link_data['images'] : array();
					$link_swatches   = isset( $link_data['swatches'] ) ? $link_data['swatches'] : array();

					// get product ids
					$link_products = array();
					$link_source   = isset( $link_data['source'] ) ? $link_data['source'] : 'products';

					if ( ( $link_source === 'products' ) && isset( $link_data['products'] ) && ! empty( $link_data['products'] ) ) {
						$link_products = explode( ',', $link_data['products'] );
					}

					if ( ( $link_source === 'categories' ) && isset( $link_data['categories'] ) && ! empty( $link_data['categories'] ) ) {
						$categories = array_map( 'trim', explode( ',', $link_data['categories'] ) );

						if ( ! empty( $categories ) ) {
							$args = array(
								'post_type'           => 'product',
								'post_status'         => 'publish',
								'ignore_sticky_posts' => 1,
								'posts_per_page'      => - 1,
								'tax_query'           => array(
									array(
										'taxonomy' => 'product_cat',
										'field'    => 'slug',
										'terms'    => $categories,
										'operator' => 'IN',
									)
								)
							);

							$products = new WP_Query( $args );

							if ( $products->have_posts() ) {
								while ( $products->have_posts() ) {
									$products->the_post();
									$link_products[] = get_the_ID();
								}
							}

							wp_reset_postdata();
						}
					}

					if ( ( $link_source === 'tags' ) && isset( $link_data['tags'] ) && ! empty( $link_data['tags'] ) ) {
						$tags = array_map( 'trim', explode( ',', $link_data['tags'] ) );

						if ( ! empty( $tags ) ) {
							$args = array(
								'post_type'           => 'product',
								'post_status'         => 'publish',
								'ignore_sticky_posts' => 1,
								'posts_per_page'      => - 1,
								'tax_query'           => array(
									array(
										'taxonomy' => 'product_tag',
										'field'    => 'slug',
										'terms'    => $tags,
										'operator' => 'IN',
									)
								)
							);

							$products = new WP_Query( $args );

							if ( $products->have_posts() ) {
								while ( $products->have_posts() ) {
									$products->the_post();
									$link_products[] = get_the_ID();
								}
							}

							wp_reset_postdata();
						}
					}

					// exclude hidden or unpurchasable
					if ( ( self::get_setting( 'exclude_hidden', 'no' ) === 'yes' ) || ( self::get_setting( 'exclude_unpurchasable', 'no' ) === 'yes' ) ) {
						foreach ( $link_products as $key => $link_product_id ) {
							$link_product = wc_get_product( $link_product_id );

							if ( ! $link_product || ( ! $link_product->is_visible() && ( self::get_setting( 'exclude_hidden', 'no' ) === 'yes' ) ) || ( ( ! $link_product->is_purchasable() || ! $link_product->is_in_stock() ) && ( self::get_setting( 'exclude_unpurchasable', 'no' ) === 'yes' ) ) ) {
								unset( $link_products[ $key ] );
							}
						}
					}

					// exclude current product
					$link_products = array_diff( $link_products, array( $product_id ) );

					if ( empty( $link_products ) ) {
						return;
					}

					$all_taxonomies             = [];
					$filter_assigned_attributes = array_filter( $_product->get_attributes(), 'wc_attributes_array_filter_visible' );
					$assigned_attributes        = array_keys( $filter_assigned_attributes );
					$product_attributes         = [];

					foreach ( $assigned_attributes as $assigned_attribute ) {
						$product_attributes[ $assigned_attribute ] = wc_get_product_terms( $product_id, $assigned_attribute, array( 'fields' => 'ids' ) );
					}

					if ( ! empty( $link_attributes ) ) {
						do_action( 'wpclv_wrap_above', $link_attributes );
						?>
                        <div class="wpclv-attributes">
							<?php
							do_action( 'wpclv_wrap_before', $link_attributes );

							$link_attributes_ids = array_map( function ( $e ) {
								return (int) filter_var( $e, FILTER_SANITIZE_NUMBER_INT );
							}, $link_attributes );

							foreach ( $link_attributes as $link_attribute ) {
								$link_attribute_id = (int) filter_var( $link_attribute, FILTER_SANITIZE_NUMBER_INT );
								$attribute         = wc_get_attribute( $link_attribute_id );
								$terms             = get_terms( $attribute->slug, array( 'hide_empty' => false ) );
								$current_terms     = wc_get_product_terms( $product_id, $attribute->slug, array( 'fields' => 'ids' ) );
								array_push( $all_taxonomies, $attribute->slug );
								?>
                                <div class="wpclv-attribute">
									<?php do_action( 'wpclv_attribute_before', $attribute ); ?>
                                    <div class="wpclv-attribute-label">
										<?php
										do_action( 'wpclv_attribute_label_before', $attribute );
										echo esc_html( $attribute->name );
										do_action( 'wpclv_attribute_label_after', $attribute );
										?>
                                    </div>
									<?php if ( ! empty( $terms ) ) { ?>
                                        <div class="wpclv-terms">
											<?php foreach ( $terms as $term ) {
												if ( in_array( $term->term_id, $current_terms ) ) {
													if ( in_array( $link_attribute, $link_images ) ) {
														self::term( 'image', $attribute, $term, true, $product_id );
													} elseif ( in_array( $link_attribute, $link_swatches ) && class_exists( 'WPCleverWpcvs' ) ) {
														self::term( 'swatches', $attribute, $term, true );
													} else {
														self::term( 'button', $attribute, $term, true, $product_id );
													}
												} else {
													$tax_query = array( 'relation' => 'AND' );

													$tax_query_ori = [
														'taxonomy' => $term->taxonomy,
														'field'    => 'id',
														'terms'    => $term->term_id
													];

													foreach ( $product_attributes as $product_attribute_key => $product_attribute ) {
														$product_attribute_id = wc_attribute_taxonomy_id_by_name( $product_attribute_key );

														if ( ! in_array( $product_attribute_id, $link_attributes_ids ) ) {
															continue;
														}

														if ( $term->taxonomy != $product_attribute_key ) {
															$tax_query[] = array(
																'taxonomy' => $product_attribute_key,
																'field'    => 'id',
																'terms'    => $product_attribute
															);
														}
													}

													array_push( $tax_query, $tax_query_ori );

													$linked_id = self::get_linked_product_id( $tax_query, $link_products );

													if ( $linked_id ) {
														if ( in_array( $link_attribute, $link_images ) ) {
															self::term( 'image', $attribute, $term, false, $linked_id );
														} elseif ( in_array( $link_attribute, $link_swatches ) && class_exists( 'WPCleverWpcvs' ) ) {
															self::term( 'swatches', $attribute, $term, false, $linked_id );
														} else {
															self::term( 'button', $attribute, $term, false, $linked_id );
														}
													} else {
														$linked_id = self::get_linked_product_id( array( $tax_query_ori ), $link_products );

														if ( $linked_id ) {
															if ( in_array( $link_attribute, $link_images ) ) {
																self::term( 'image', $attribute, $term, false, $linked_id );
															} elseif ( in_array( $link_attribute, $link_swatches ) && class_exists( 'WPCleverWpcvs' ) ) {
																self::term( 'swatches', $attribute, $term, false, $linked_id );
															} else {
																self::term( 'button', $attribute, $term, false, $linked_id );
															}
														} elseif ( self::get_setting( 'hide_empty', 'yes' ) === 'no' ) {
															if ( in_array( $link_attribute, $link_images ) ) {
																self::term( 'image', $attribute, $term, false );
															} elseif ( in_array( $link_attribute, $link_swatches ) && class_exists( 'WPCleverWpcvs' ) ) {
																self::term( 'swatches', $attribute, $term, false );
															} else {
																self::term( 'button', $attribute, $term, false );
															}
														}
													}
												}
											} ?>
                                        </div>
									<?php } ?>
									<?php do_action( 'wpclv_attribute_after', $attribute ); ?>
                                </div>
							<?php }

							do_action( 'wpclv_wrap_after', $link_attributes );
							?>
                        </div>
						<?php
						do_action( 'wpclv_wrap_below', $link_attributes );
					}
				}

				public static function get_linked_data( $product_id ) {
					// new data, get the ID only
					$new_links = get_posts( array(
						'post_type'      => 'wpclv',
						'post_status'    => 'publish',
						'posts_per_page' => - 1,
						'fields'         => 'ids'
					) );

					if ( ! empty( $new_links ) ) {
						foreach ( $new_links as $new_link ) {
							$link = get_post_meta( $new_link, 'wpclv_link', true );

							if ( ! empty( $link ) ) {
								$link_source = isset( $link['source'] ) ? $link['source'] : 'products';

								if ( ( $link_source === 'products' ) && isset( $link['products'] ) && ! empty( $link['products'] ) ) {
									$product_ids = explode( ',', $link['products'] );

									if ( in_array( $product_id, $product_ids ) ) {
										return $link;
									}
								}

								if ( ( $link_source === 'categories' ) && isset( $link['categories'] ) && ! empty( $link['categories'] ) ) {
									$categories = array_map( 'trim', explode( ',', $link['categories'] ) );

									if ( has_term( $categories, 'product_cat', $product_id ) ) {
										return $link;
									}
								}

								if ( ( $link_source === 'tags' ) && isset( $link['tags'] ) && ! empty( $link['tags'] ) ) {
									$tags = array_map( 'trim', explode( ',', $link['tags'] ) );

									if ( has_term( $tags, 'product_tag', $product_id ) ) {
										return $link;
									}
								}

								if ( ! in_array( $link_source, array(
										'products',
										'categories',
										'tags'
									) ) && isset( $link['terms'] ) && ! empty( $link['terms'] ) ) {
									$terms = array_map( 'trim', explode( ',', $link['terms'] ) );

									if ( has_term( $terms, $link_source, $product_id ) ) {
										return $link;
									}
								}
							}
						}
					}

					// old data
					if ( is_array( self::$links ) && ( count( self::$links ) > 0 ) ) {
						foreach ( self::$links as $link ) {
							$link_source = isset( $link['source'] ) ? $link['source'] : 'products';

							if ( ( $link_source === 'products' ) && isset( $link['products'] ) && ! empty( $link['products'] ) ) {
								$product_ids = explode( ',', $link['products'] );

								if ( in_array( $product_id, $product_ids ) ) {
									return $link;
								}
							}

							if ( ( $link_source === 'categories' ) && isset( $link['categories'] ) && ! empty( $link['categories'] ) ) {
								$categories = array_map( 'trim', explode( ',', $link['categories'] ) );

								if ( has_term( $categories, 'product_cat', $product_id ) ) {
									return $link;
								}
							}

							if ( ( $link_source === 'tags' ) && isset( $link['tags'] ) && ! empty( $link['tags'] ) ) {
								$tags = array_map( 'trim', explode( ',', $link['tags'] ) );

								if ( has_term( $tags, 'product_tag', $product_id ) ) {
									return $link;
								}
							}
						}
					}

					return false;
				}

				// return post id
				public static function get_linked_product_id( $tax_query, $link_products = [], $order = 'ASC' ) {
					$args = [
						'post_type'      => 'product',
						'posts_per_page' => 1,
						'order'          => $order,
						'fields'         => 'ids'
					];

					if ( $tax_query ) {
						$args['tax_query'] = $tax_query;
					}

					if ( $link_products ) {
						$args['post__in'] = $link_products;
					}

					$filter_product = get_posts( $args );

					if ( $filter_product ) {
						return $filter_product[0];
					} else {
						return false;
					}
				}

				function action_links( $links, $file ) {
					static $plugin;

					if ( ! isset( $plugin ) ) {
						$plugin = plugin_basename( __FILE__ );
					}

					if ( $plugin === $file ) {
						$settings         = '<a href="' . admin_url( 'admin.php?page=wpclever-wpclv&tab=settings' ) . '">' . esc_html__( 'Settings', 'wpc-linked-variation' ) . '</a>';
						$linked           = '<a href="' . admin_url( 'admin.php?page=wpclever-wpclv&tab=linked' ) . '">' . esc_html__( 'Linked Variations', 'wpc-linked-variation' ) . '</a>';
						$links['premium'] = '<a href="' . admin_url( 'admin.php?page=wpclever-wpclv&tab=premium' ) . '">' . esc_html__( 'Premium Version', 'wpc-linked-variation' ) . '</a>';
						array_unshift( $links, $settings, $linked );
					}

					return (array) $links;
				}

				function row_meta( $links, $file ) {
					static $plugin;

					if ( ! isset( $plugin ) ) {
						$plugin = plugin_basename( __FILE__ );
					}

					if ( $plugin === $file ) {
						$row_meta = array(
							'support' => '<a href="' . esc_url( WPCLV_DISCUSSION ) . '" target="_blank">' . esc_html__( 'Community support', 'wpc-linked-variation' ) . '</a>',
						);

						return array_merge( $links, $row_meta );
					}

					return (array) $links;
				}

				function search_term() {
					$return = array();

					$args = array(
						'taxonomy'   => sanitize_text_field( $_REQUEST['taxonomy'] ),
						'orderby'    => 'id',
						'order'      => 'ASC',
						'hide_empty' => false,
						'fields'     => 'all',
						'name__like' => sanitize_text_field( $_REQUEST['q'] ),
					);

					$terms = get_terms( $args );

					if ( count( $terms ) ) {
						foreach ( $terms as $term ) {
							$return[] = array( $term->slug, $term->name );
						}
					}

					wp_send_json( $return );
				}

				function wpcsm_locations( $locations ) {
					$locations['WPC Linked Variation'] = array(
						'wpclv_wrap_above'             => esc_html__( 'Before container', 'wpc-linked-variation' ),
						'wpclv_wrap_below'             => esc_html__( 'After container', 'wpc-linked-variation' ),
						'wpclv_wrap_before'            => esc_html__( 'Before attributes', 'wpc-linked-variation' ),
						'wpclv_wrap_after'             => esc_html__( 'After attributes', 'wpc-linked-variation' ),
						'wpclv_attribute_before'       => esc_html__( 'Before attribute', 'wpc-linked-variation' ),
						'wpclv_attribute_after'        => esc_html__( 'After attribute', 'wpc-linked-variation' ),
						'wpclv_attribute_label_before' => esc_html__( 'Before attribute label', 'wpc-linked-variation' ),
						'wpclv_attribute_label_after'  => esc_html__( 'After attribute label', 'wpc-linked-variation' ),
					);

					return $locations;
				}
			}

			return WPCleverWpclv::instance();
		}
	}
}

if ( ! function_exists( 'wpclv_notice_wc' ) ) {
	function wpclv_notice_wc() {
		?>
        <div class="error">
            <p><strong>WPC Linked Variation</strong> requires WooCommerce version 3.0 or greater.</p>
        </div>
		<?php
	}
}
